/*-------------------------------------------------------------------------
 *
 * param_data.cpp
 *
 *         : BSD.
 *         Copyright (c) 2004 Andrei V. Shetuhin
 *
 *             
 *
 *
 *-------------------------------------------------------------------------
 */
#include <ctpp/param_data.hpp>

namespace template_parser_ns
{
//
// 
//
param_data::param_data(e_value_type eIType): parent(NULL), type(eIType), value(NULL), undef(true)
{
	switch (type)
	{
		case VAR:
			{ t_param_str * pVariable = new t_param_str;     value = static_cast<void *>(pVariable); }
//cerr << "VAR" << endl;
			break;
		case ARRAY:
			{ t_param_array * pVariable = new t_param_array; value = static_cast<void *>(pVariable); }
//cerr << "ARRAY" << endl;
			break;
		case HASH:
			{ t_param_hash * pVariable = new t_param_hash;   value = static_cast<void *>(pVariable); }
//cerr << "HASH" << endl;
			break;
		default:
			throw std::logic_error("Unknown variable type!");
	}
}
//
// 
//
param_data::~param_data() throw()
{
	try
	{
		switch (type)
		{
			// 
			case VAR:
				{ std::string * pVar = static_cast<t_param_str *>(value); delete pVar; }
//cerr << "~VAR" << endl;
				break;
			// 
			case ARRAY:
				{
					t_param_array * pVar = static_cast<t_param_array *>(value);
					t_param_array::iterator itArr = pVar -> begin();
					while (itArr != pVar -> end()) { delete *itArr; itArr++; }
					delete pVar;
				}
//cerr << "~ARRAY" << endl;
				break;
			// 
			case HASH:
				{
					t_param_hash * pVar = static_cast<t_param_hash *>(value);
					t_param_hash::iterator itArr = pVar -> begin();
					while (itArr != pVar -> end()) { delete (*itArr).second; itArr++; }
					delete pVar;
				}
//cerr << "~HASH" << endl;
				break;
			//    
			default:
				;; // !
		}
	}
	catch(...) { ;; }
}

//
//     VAL
//
t_param_str * param_data::val()
{
	//  - ?
	if (type == VAR) { return static_cast<t_param_str *>(value); }
	// - 
	throw std::logic_error("ValType is not VAL");
}

//
//     ARRAY
//
t_param_array * param_data::array()
{
	//  - ?
	if (type == ARRAY) { return static_cast<t_param_array *>(value); }
	// - 
	throw std::logic_error("ValType is not ARRAY");
}

//
//     HASH
//
t_param_hash * param_data::hash()
{
	//  - ?
	if (type == HASH) { return static_cast<t_param_hash *>(value); }
	// - 
	throw std::logic_error("ValType is not HASH");
}

//
//  
//
param_data::e_value_type param_data::get_value_type() { return type; }

//
//   VALUE  HASH
//
param_data * param_data::hash_insert_new_var(const std::string &sKey, const std::string &sValue)
{
	param_data* pVar;
	if(type == HASH)
	{
		pVar = new param_data(VAR);
		pVar -> val() -> assign(sValue);
		//    ,  
		//    -- 
		t_param_hash::iterator itmHash = hash() -> find(sKey);
		if (itmHash == hash() -> end()) { hash() -> insert(std::pair<std::string, param_data *>(sKey, pVar)); }
		else
		{
			delete itmHash -> second;
			itmHash -> second = pVar;
		}
	}
	else { throw std::logic_error("ValType is not HASH"); }
return pVar;
}

//
//    
//
param_data * param_data::hash_insert_new_array(const std::string &sKey)
{
	param_data* pArr = NULL;
	if(type == HASH)
	{
		pArr = new param_data(ARRAY);
		//    ,  
		//    -- 
		t_param_hash::iterator itmHash = hash() -> find(sKey);
		if (itmHash == hash() -> end()) { hash() -> insert(std::pair<std::string, param_data *>(sKey, pArr)); }
		else
		{
			delete itmHash -> second;
			itmHash -> second = pArr;
		}
	}
	else { throw std::logic_error("ValType is not HASH"); }
return pArr;
}

//
//    
//
param_data * param_data::array_insert_new_var(const std::string &sKey)
{
	param_data* pVar = NULL;
	if(type == ARRAY)
	{
		pVar = new param_data(VAR);
		pVar -> val() -> assign(sKey);
		array() -> push_back(pVar);
	}
	else { throw std::logic_error("ValType is not ARRAY"); }
return pVar;
}

//
//    
//
param_data * param_data::array_insert_new_hash()
{
	param_data* pHash = NULL;
	if(type == ARRAY)
	{
		pHash = new param_data(HASH);
		array() -> push_back(pHash);
	}
	else { throw std::logic_error("ValType is not ARRAY"); }
return pHash;
}

//
//    
//
void param_data::hash_erase_var(const std::string &sKey)
{
	if(type == HASH)
	{
		//    ,  
		//    -- 
		t_param_hash::iterator itmHash = hash() -> find(sKey);
		if (itmHash != hash() -> end())
		{
			delete itmHash -> second;
			hash() -> erase(itmHash);
		}
	}
	else { throw std::logic_error("ValType is not HASH"); }
}


} // namespace template_parser_ns
// End.
